# Generated by roxygen2: do not edit by hand

S3method(Anova,resi)
S3method(anova,resi)
S3method(ggplot,anova_resi)
S3method(ggplot,resi)
S3method(ggplot,summary_resi)
S3method(plot,anova_resi)
S3method(plot,resi)
S3method(plot,summary_resi)
S3method(print,omnibus_resi)
S3method(print,resi)
S3method(print,summary_resi)
S3method(resi,coxph)
S3method(resi,default)
S3method(resi,gee)
S3method(resi,geeglm)
S3method(resi,glm)
S3method(resi,glmgee)
S3method(resi,glmmTMB)
S3method(resi,hurdle)
S3method(resi,lm)
S3method(resi,lme)
S3method(resi,lmerMod)
S3method(resi,nls)
S3method(resi,survreg)
S3method(resi,zeroinfl)
S3method(resi_pe,coxph)
S3method(resi_pe,default)
S3method(resi_pe,emmGrid)
S3method(resi_pe,gee)
S3method(resi_pe,geeglm)
S3method(resi_pe,glm)
S3method(resi_pe,glmgee)
S3method(resi_pe,glmmTMB)
S3method(resi_pe,hurdle)
S3method(resi_pe,lm)
S3method(resi_pe,lme)
S3method(resi_pe,lmerMod)
S3method(resi_pe,nls)
S3method(resi_pe,survreg)
S3method(resi_pe,zeroinfl)
S3method(summary,resi)
export(Rsq2S)
export(S2Rsq)
export(S2chisq)
export(S2d)
export(S2fsq)
export(S2z)
export(chisq2S)
export(d2S)
export(f2S)
export(fsq2S)
export(omnibus)
export(resi)
export(resi_pe)
export(t2S)
export(z2S)
import(ggplot2)
importFrom(aod,wald.test)
importFrom(boot,boot)
importFrom(car,Anova)
importFrom(clubSandwich,vcovCR)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,strwidth)
importFrom(lmtest,waldtest)
importFrom(nlme,getGroups)
importFrom(sandwich,vcovHC)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,capture.output)
importFrom(utils,methods)
