
<!-- README.md is generated from README.Rmd. Please edit that file -->

# RESI <img src="man/figures/logo.png" align="right" height="138"/>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/RESI)](https://CRAN.R-project.org/package=RESI)
[![Website](https://img.shields.io/badge/Website-RESI-blue)](https://statimagcoll.github.io/RESI/)

<!-- badges: end -->

RESI is an R package designed to implement the Robust Effect Size Index
(RESI, denoted as S) described in Vandekar, Tao, & Blume (2020). The
RESI is a versatile effect size measure that can be easily computed and
added to common reports (such as summary and ANOVA tables). This package
currently supports <code>lm</code>, <code>glm</code>, <code>nls</code>,
<code>survreg</code>, <code>coxph</code>, <code>hurdle</code>,
<code>zeroinfl</code>, <code>gee</code>, <code>geeglm</code>,
<code>lme</code>, <code>lmerMod</code>, <code>lmrob</code>, and
<code>glmrob</code> models. Confidence intervals are now computed using
the bootstrap or one of three asymptotic methods: a profiled quadratic
form, Cornish-Fisher expansion, or normal approximation. A Bayesian
bootstrap is also available for <code>lm</code> and <code>nls</code>
models. In addition to the main <code>resi</code> function, the package
also includes a point-estimate-only function (<code>resi_pe</code>),
conversions from S to other common effect size measures and vice versa,
print methods, plot methods, summary methods, and Anova/anova methods.

If you would like to contribute to the package, please branch off of our
[GitHub](https://github.com/statimagcoll/RESI) and submit a pull request
describing the contribution. Please use the [GitHub
Issues](https://github.com/statimagcoll/RESI/issues) page to report any
problems and the
[Discussions](https://github.com/statimagcoll/RESI/discussions/categories/q-a)
page to seek additional support.

## References

Jones M, Kang K, Vandekar S (2025). RESI: An R Package for Robust Effect
Sizes. <i> Journal of Statistical Software, 112(3), 1–27. <i>
<https://doi.org/10.18637/jss.v112.i03>.

Kang K, Jones MT, Armstrong K, Avery S, McHugo M, Heckers S, & Vandekar
S. Accurate Confidence and Bayesian Interval Estimation for
Non-centrality Parameters and Effect Size Indices. <i>Psychometrika.</i>
2023. <https://doi.org/10.1007/s11336-022-09899-x>.

Kang K, Seidlitz J, Bethlehem RAI, et al. Study design features increase
replicability in brain-wide association studies. <i>Nature.</i> 2024.
<https://doi.org/10.1038/s41586-024-08260-9>.

Vandekar S, Tao R, Blume J. A Robust Effect Size Index.
<i>Psychometrika.</i> 2020;85(1):232–246.
<https://doi.org/10.1007/s11336-020-09698-2>.

Zhang X, Muscatello R, Jones M, Corbett B, Vandekar S. Asymptotic
Distribution of Robust Effect Size Index. <i>arXiv:2601.19004.</i> 2026.
<https://doi.org/10.48550/arXiv.2601.19004>.
